]> git.tuebingen.mpg.de Git - paraslash.git/blobdiff - mp4.c
mp4: Make most loop variables unsigned.
[paraslash.git] / mp4.c
diff --git a/mp4.c b/mp4.c
index 3963f7830fcb67e9eb2a8ec0aa7cab43779f9693..9ed0fcbb4dfbfaa07b53893e7e021607a17636ad 100644 (file)
--- a/mp4.c
+++ b/mp4.c
@@ -75,8 +75,8 @@ struct mp4 {
 };
 
 /*
* Returns -1, 0, or 1 on errors/EOF/success. Partial reads followed by EOF or
* read errors are treated as errors.
  */
 static int read_data(struct mp4 *f, void *data, size_t size)
 {
@@ -86,7 +86,7 @@ static int read_data(struct mp4 *f, void *data, size_t size)
                        continue;
                /* regard EAGAIN as an error as reads should be blocking. */
                if (ret <= 0)
-                       return ret < 0? -1 : 0;
+                       return ret < 0? -E_MP4_READ : 0;
                size -= ret;
        }
        return 1;
@@ -188,8 +188,7 @@ static int atom_read_header(struct mp4 *f, uint8_t *atom_type,
        } else {
                if (header_size)
                        *header_size = 8;
-               if (atom_size)
-                       *atom_size = size;
+               *atom_size = size;
        }
        *atom_type = atom_name_to_type(atom_header + 4);
        return 1;
@@ -213,7 +212,6 @@ static void skip_bytes(struct mp4 *f, off_t num_skip)
 static int read_stsz(struct mp4 *f)
 {
        int ret;
-       int32_t i;
        struct mp4_track *t = &f->track;
 
        if (t->state != ATS_SEEN_MP4A || t->stsz_table)
@@ -228,8 +226,8 @@ static int read_stsz(struct mp4 *f)
        if (t->stsz_sample_size != 0)
                return 1;
        t->stsz_table = para_malloc(t->stsz_sample_count * sizeof(int32_t));
-       for (i = 0; i < t->stsz_sample_count; i++) {
-               ret = read_int32(f, &t->stsz_table[i]);
+       for (uint32_t n = 0; n < t->stsz_sample_count; n++) {
+               ret = read_int32(f, &t->stsz_table[n]);
                if (ret <= 0)
                        return ret;
        }
@@ -239,7 +237,6 @@ static int read_stsz(struct mp4 *f)
 static int read_stts(struct mp4 *f)
 {
        int ret;
-       int32_t i;
        struct mp4_track *t = &f->track;
 
        if (t->state != ATS_SEEN_MP4A || t->stts_sample_count)
@@ -250,8 +247,8 @@ static int read_stts(struct mp4 *f)
                return ret;
        t->stts_sample_count = para_malloc(t->stts_entry_count
                * sizeof(int32_t));
-       for (i = 0; i < t->stts_entry_count; i++) {
-               ret = read_int32(f, &t->stts_sample_count[i]);
+       for (uint32_t n = 0; n < t->stts_entry_count; n++) {
+               ret = read_int32(f, &t->stts_sample_count[n]);
                if (ret <= 0)
                        return ret;
                skip_bytes(f, 4); /* sample delta */
@@ -262,7 +259,6 @@ static int read_stts(struct mp4 *f)
 static int read_stsc(struct mp4 *f)
 {
        int ret;
-       int32_t i;
        struct mp4_track *t = &f->track;
 
        if (t->state != ATS_SEEN_MP4A)
@@ -276,11 +272,11 @@ static int read_stsc(struct mp4 *f)
        t->stsc_first_chunk = para_malloc(t->stsc_entry_count * sizeof(int32_t));
        t->stsc_samples_per_chunk = para_malloc(t->stsc_entry_count
                * sizeof (int32_t));
-       for (i = 0; i < t->stsc_entry_count; i++) {
-               ret = read_int32(f, &t->stsc_first_chunk[i]);
+       for (uint32_t n = 0; n < t->stsc_entry_count; n++) {
+               ret = read_int32(f, &t->stsc_first_chunk[n]);
                if (ret <= 0)
                        return ret;
-               ret = read_int32(f, &t->stsc_samples_per_chunk[i]);
+               ret = read_int32(f, &t->stsc_samples_per_chunk[n]);
                if (ret <= 0)
                        return ret;
                skip_bytes(f, 4); /* sample desc index */
@@ -291,7 +287,6 @@ static int read_stsc(struct mp4 *f)
 static int read_stco(struct mp4 *f)
 {
        int ret;
-       int32_t i;
        struct mp4_track *t = &f->track;
 
        if (t->state != ATS_SEEN_MP4A || t->stco_chunk_offset)
@@ -302,8 +297,8 @@ static int read_stco(struct mp4 *f)
                return ret;
        t->stco_chunk_offset = para_malloc(t->stco_entry_count
                * sizeof(int32_t));
-       for (i = 0; i < t->stco_entry_count; i++) {
-               ret = read_int32(f, &t->stco_chunk_offset[i]);
+       for (uint32_t n = 0; n < t->stco_entry_count; n++) {
+               ret = read_int32(f, &t->stco_chunk_offset[n]);
                if (ret <= 0)
                        return ret;
        }
@@ -313,7 +308,7 @@ static int read_stco(struct mp4 *f)
 static int read_stsd(struct mp4 *f)
 {
        int ret;
-       uint32_t i, entry_count;
+       uint32_t entry_count;
 
        if (f->track.state != ATS_INITIAL)
                return 1;
@@ -321,7 +316,7 @@ static int read_stsd(struct mp4 *f)
        ret = read_int32(f, &entry_count);
        if (ret <= 0)
                return ret;
-       for (i = 0; i < entry_count; i++) {
+       for (uint32_t n = 0; n < entry_count; n++) {
                uint64_t skip = get_position(f);
                uint64_t size;
                uint8_t atom_type = 0;
@@ -381,7 +376,7 @@ static int parse_tag(struct mp4 *f, uint8_t parent, int32_t size)
                if (atom_type != ATOM_DATA)
                        continue;
                skip_bytes(f, 8); /* version (1), flags (3), reserved (4) */
-               ret = -ERRNO_TO_PARA_ERROR(EINVAL);
+               ret = -E_MP4_CORRUPT;
                if (subsize < header_size + 8 || subsize > UINT_MAX)
                        goto fail;
                len = subsize - (header_size + 8);
@@ -393,7 +388,7 @@ static int parse_tag(struct mp4 *f, uint8_t parent, int32_t size)
                value[len] = '\0';
        }
        if (!value)
-               return -ERRNO_TO_PARA_ERROR(EINVAL);
+               return -E_MP4_CORRUPT;
        f->meta.tags = para_realloc(f->meta.tags, (f->meta.count + 1)
                * sizeof(struct mp4_tag));
        tag = f->meta.tags + f->meta.count;
@@ -539,7 +534,7 @@ static int parse_sub_atoms(struct mp4 *f, uint64_t total_size, bool meta_only)
                if (ret <= 0)
                        return ret;
                if (size == 0)
-                       return -1;
+                       return -E_MP4_CORRUPT;
                dest = get_position(f) + size - header_size;
                if (atom_type == ATOM_TRAK && f->track.state == ATS_SEEN_MP4A) {
                        f->track.state = ATS_TRACK_CHANGE;
@@ -592,10 +587,8 @@ static int open_file(const struct mp4_callback *cb, bool meta_only, struct mp4 *
                if (ret <= 0)
                        break;
        }
-       if (ret < 0) {
-               ret = -E_MP4_OPEN;
+       if (ret < 0)
                goto fail;
-       }
        ret = -E_MP4_TRACK;
        if (f->track.channel_count == 0)
                goto fail;
@@ -614,16 +607,14 @@ int mp4_open_read(const struct mp4_callback *cb, struct mp4 **result)
 
 void mp4_close(struct mp4 *f)
 {
-       int32_t i;
-
        free(f->track.stsz_table);
        free(f->track.stts_sample_count);
        free(f->track.stsc_first_chunk);
        free(f->track.stsc_samples_per_chunk);
        free(f->track.stco_chunk_offset);
-       for (i = 0; i < f->meta.count; i++) {
-               free(f->meta.tags[i].item);
-               free(f->meta.tags[i].value);
+       for (uint32_t n = 0; n < f->meta.count; n++) {
+               free(f->meta.tags[n].item);
+               free(f->meta.tags[n].value);
        }
        free(f->meta.tags);
        free(f);
@@ -634,15 +625,15 @@ static int32_t chunk_of_sample(const struct mp4 *f, int32_t sample,
 {
        const struct mp4_track *t = &f->track;
        uint32_t *fc = t->stsc_first_chunk, *spc = t->stsc_samples_per_chunk;
-       int32_t chunk1, chunk1samples, n, total, i;
+       uint32_t chunk1, chunk1samples, n, total, k;
 
-       for (i = 1, total = 0; i < t->stsc_entry_count; i++, total += n) {
-               n = (fc[i] - fc[i - 1]) * spc[i - 1]; /* number of samples */
+       for (k = 1, total = 0; k < t->stsc_entry_count; k++, total += n) {
+               n = (fc[k] - fc[k - 1]) * spc[k - 1]; /* number of samples */
                if (sample < total + n)
                        break;
        }
-       chunk1 = fc[i - 1];
-       chunk1samples = spc[i - 1];
+       chunk1 = fc[k - 1];
+       chunk1samples = spc[k - 1];
        if (chunk1samples != 0)
                *chunk = (sample - total) / chunk1samples + chunk1;
        else
@@ -711,11 +702,10 @@ uint32_t mp4_get_channel_count(const struct mp4 *f)
 int32_t mp4_num_samples(const struct mp4 *f)
 {
        const struct mp4_track *t = &f->track;
-       int32_t i;
        int32_t total = 0;
 
-       for (i = 0; i < t->stts_entry_count; i++)
-               total += t->stts_sample_count[i];
+       for (uint32_t n = 0; n < t->stts_entry_count; n++)
+               total += t->stts_sample_count[n];
        return total;
 }
 
@@ -899,70 +889,21 @@ free_moov:
        return ret;
 }
 
-static char *meta_find_by_name(const struct mp4 *f, const char *item)
-{
-       uint32_t i;
-
-       for (i = 0; i < f->meta.count; i++)
-               if (!strcasecmp(f->meta.tags[i].item, item))
-                       return para_strdup(f->meta.tags[i].value);
-       return NULL;
-}
-
 /**
- * Return the value of the artist meta tag of an mp4 file.
+ * Return the value of the given tag item.
  *
  * \param f Must not be NULL.
+ * \param item "artist", "title", "album", "comment", or "date".
  *
- * \return If the file does not contain this metadata tag, the function returns
- * NULL. Otherwise, a copy of the tag value is returned. The caller should free
- * this memory when it is no longer needed.
+ * \return The function always returns NULL if the given item is not in the
+ * above list. Otherwise, if the file does not contain a tag for the given
+ * item, the function also returns NULL. Otherwise a copy of the tag value is
+ * returned and the caller should free this memory when it is no longer needed.
  */
-char *mp4_meta_get_artist(const struct mp4 *f)
+char *mp4_get_tag_value(const struct mp4 *f, const char *item)
 {
-       return meta_find_by_name(f, "artist");
-}
-
-/**
- * Return the value of the title meta tag of an mp4 file.
- *
- * \param f See \ref mp4_meta_get_artist().
- * \return See \ref mp4_meta_get_artist().
- */
-char *mp4_meta_get_title(const struct mp4 *f)
-{
-       return meta_find_by_name(f, "title");
-}
-
-/**
- * Return the value of the date meta tag of an mp4 file.
- *
- * \param f See \ref mp4_meta_get_artist().
- * \return See \ref mp4_meta_get_artist().
- */
-char *mp4_meta_get_date(const struct mp4 *f)
-{
-       return meta_find_by_name(f, "date");
-}
-
-/**
- * Return the value of the album meta tag of an mp4 file.
- *
- * \param f See \ref mp4_meta_get_artist().
- * \return See \ref mp4_meta_get_artist().
- */
-char *mp4_meta_get_album(const struct mp4 *f)
-{
-       return meta_find_by_name(f, "album");
-}
-
-/**
- * Return the value of the comment meta tag of an mp4 file.
- *
- * \param f See \ref mp4_meta_get_artist().
- * \return See \ref mp4_meta_get_artist().
- */
-char *mp4_meta_get_comment(const struct mp4 *f)
-{
-       return meta_find_by_name(f, "comment");
+       for (unsigned n = 0; n < f->meta.count; n++)
+               if (!strcasecmp(f->meta.tags[n].item, item))
+                       return para_strdup(f->meta.tags[n].value);
+       return NULL;
 }
@@ -75,8 +75,8 @@ struct mp4 {
 
 /*
- * Returns -1, 0, or 1 on errors/EOF/success. Partial reads followed by EOF or
- * read errors are treated as errors.
+ * Returns -E_MP4_READ, 0, or 1 on errors/EOF/success. Partial reads followed
+ * by EOF or read errors are treated as errors.
  */
 static int read_data(struct mp4 *f, void *data, size_t size)